@@ -1,6 +1,6 @@
 import numpy as np
 from miditoolkit.midi import containers
-from .data import get_heat_maps, get_drum_maps
+from .data import get_heat_maps, drum_names_to_pitches
 from .metrics import rhythm_similarity
 from .timepoints import get_rhythm_markers_by_beat
 
@@ -102,14 +102,50 @@ class RhythmKMeans:
 
 
 class DrumRawClassifier:
-    def __init__(self):
-        self.rhythms = np.zeros((0, 96))
+    def __init__(self, n_bins=96, drums=None):
+        if drums is None:
+            self.drums = ["bass_drum", "closed_hihat", "snare"]
+        else:
+            self.drums = drums
+        self.n_bins = n_bins
+        self.rhythms = dict()
 
-    def fit(self, dataset, k=64, indices=None, quantize=True):
+    def get_drum_maps(self, midi_obj, bar_resolution=1920):
+        return np.stack([get_heat_maps(
+            midi_obj=midi_obj,
+            n_bins=self.n_bins,
+            beat_resolution=bar_resolution,
+            is_drum=True,
+            pitches=drum_names_to_pitches[drum]
+        ) for drum in self.drums], axis=1)
+
+    def get_drum_dataset(self, midi_objs):
+        dataset = np.zeros((0, len(self.drums), self.n_bins))
+        for midi_obj in midi_objs:
+            time_signatures = midi_obj.time_signature_changes
+            if len(time_signatures) > 1:
+                continue
+            num = time_signatures[0].numerator
+            while num % 2 == 0:
+                num /= 2
+            if num != 1:
+                continue
+
+            drum_maps = self.get_drum_maps(
+                midi_obj=midi_obj,
+                bar_resolution=midi_obj.ticks_per_beat * 4
+            )
+            dataset = np.concatenate((dataset, drum_maps))
+
+        return dataset
+
+    def fit(self, dataset, k=100, drum="all", quantize=True):
         dataset_temp = dataset.copy()
 
-        if indices is None:
+        if drum == "all":
             indices = np.arange(dataset_temp.shape[1])
+        else:
+            indices = np.array([self.drums.index(drum)])
 
         if quantize:
             for i in np.arange(16):
@@ -124,13 +160,35 @@ class DrumRawClassifier:
                 rhythm_count[rhythm_tuple] = 1
 
         pairs = sorted(list(rhythm_count.items()), key=lambda x: -x[1])
-        self.rhythms = np.array([pair[0] for pair in pairs])[:k]
+        self.rhythms[drum] = np.array([pair[0] for pair in pairs])[:k]
 
-    def add_bar_class(self, midi_obj, bar_resolution=1920, drums=None):
-        drum_maps = get_drum_maps(midi_obj, bar_resolution=bar_resolution, drums=drums)
+    def fit_from_midi(self, midi_objs, k_all=100, k_separate=20, quantize=True):
+        dataset = self.get_drum_dataset(midi_objs)
+        self.fit(dataset, k=k_all, quantize=quantize)
+        for drum in self.drums:
+            self.fit(dataset, k=k_separate, drum=drum, quantize=quantize)
 
-        for bar, drum_map in enumerate(drum_maps.reshape(drum_maps.shape[0], len(drums) * drum_maps.shape[2])):
-            rhythm_type = np.argmax(rhythm_similarity(drum_map, self.rhythms))
+    def add_bar_class(self, midi_obj, bar_resolution=1920, rid_empty=True):
+        drum_maps = self.get_drum_maps(midi_obj, bar_resolution=bar_resolution)
+
+        for bar, drum_map in enumerate(drum_maps.reshape(drum_maps.shape[0], len(self.drums) * self.n_bins)):
+            rhythm_type = np.argmax(rhythm_similarity(drum_map, self.rhythms["all"]))
+            if rid_empty:
+                if np.sum(self.rhythms["all"][rhythm_type]) == 0.:
+                    continue
             marker = containers.Marker(text=f'drum rhythm {int(rhythm_type)}',
                                        time=bar * bar_resolution)
             midi_obj.markers.append(marker)
+
+    def add_separate_bar_class(self, midi_obj, bar_resolution=1920, rid_empty=True):
+        drum_maps = self.get_drum_maps(midi_obj, bar_resolution=bar_resolution)
+
+        for i_drum, drum in enumerate(self.drums):
+            for bar, drum_map in enumerate(drum_maps[:, i_drum, :].reshape(drum_maps.shape[0], self.n_bins)):
+                rhythm_type = np.argmax(rhythm_similarity(drum_map, self.rhythms[drum]))
+                if rid_empty:
+                    if np.sum(self.rhythms[drum][rhythm_type]) == 0.:
+                        continue
+                marker = containers.Marker(text=f'{drum} rhythm {int(rhythm_type)}',
+                                           time=bar * bar_resolution)
+                midi_obj.markers.append(marker)
