@@ -16,11 +16,13 @@ def notes_weight(notes):
     return pitch_weight(pitches) + velocity_weight(velocities)
 
 
-def get_heat_map(notes, n_bins=24, beat_resolution=480, min_pitch=0, max_pitch=127):
+def get_heat_map(notes, n_bins=24, beat_resolution=480, is_drum=False, pitches=range(0, 128)):
     bin_resolution = np.ceil(beat_resolution / n_bins).astype(int)
-    notes = [note for note in notes if min_pitch <= note.pitch <= max_pitch]
+    notes = [note for note in notes if note.pitch in pitches and note.instrument.is_drum == is_drum]
     heat_map = np.zeros(n_bins)
     weights = notes_weight(notes)
+    if is_drum:
+        weights = np.clip(np.ceil(weights), 0., 1.)
     for note, weight in zip(notes, weights):
         heat_map[(note.start % beat_resolution) // bin_resolution] += weight
 
